<?php

namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class Pla03300LimitePerCapitaPorServico extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
    protected $dataSourceManager;
    protected $util;
    protected $connection;
    protected $RestrContrService;
    protected $errorMessages = array();
    protected $errors = array();

    public function __construct($crudService, $parameterBag, $dataSourceManager, $util, $connection, $RestrContrService) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->util = $util;
        $this->connection = $connection;
        $this->RestrContrService = $RestrContrService;
    }

    //Cascade save Vigencia->Servico->Produto
    public function saveVigencia($vigencia, $response, $dataSourceName = "viglimpesergr") {
        try {
            $vigencia["NRVIGENCIA"] = $this->RestrContrService->getNewCode($dataSourceName);
            $dataSet = new DataSet($dataSourceName, array($vigencia));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrvigencia = $rowsToSave[0]["NRVIGENCIA"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrvigencia;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveServico($servico, $response) {
        try {
            $dataSourceName = "limpesergr";
            $servico["NRLIMPESERGR"] = $this->RestrContrService->getNewCode($dataSourceName);
            $dataSet = new DataSet($dataSourceName, array($servico));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrlimpesergr = $rowsToSave[0]["NRLIMPESERGR"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrlimpesergr;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveProduto(DTO\Request $request, DTO\Response $response) {
        try {
            
            $row = $request->getRow();
            $this->connection->beginTransaction();
            if (isset($row["vigencia"])) {
                $nrvigencia = $this->saveVigencia($row["vigencia"], $response);
            }
            if (isset($row["servico"])) {
                $row["servico"]["NRVIGENCIA"] = isset($nrvigencia) ? $nrvigencia : $row["servico"]["NRVIGENCIA"];
                $nrlimpesergr = $this->saveServico($row["servico"], $response);
            }
            $this->dataSourceName = "itlimpesergr";
            foreach ($row["produto"]["NMPRODUTO"] as $produto) {
                $produtoRow = array(
                    "NRITLIMPESERGR"   => $this->RestrContrService->getNewCode($this->dataSourceName),
                    "NRLIMPESERGR"     => isset($nrlimpesergr) ? $nrlimpesergr : $row["produto"]["NRLIMPESERGR"],
                    "NRGRUPOPRATOSERV" => $row["produto"]["NRGRUPOPRATOSERV"],
                    "NRGRUPOPRATO"     => $row["produto"]["NRGRUPOPRATO"],
                    "QTMAXLIMPERSERGR" => $row["produto"]["QTMAXLIMPERSERGR"],
                    "QTMINLIMPERSERGR" => $row["produto"]["QTMINLIMPERSERGR"],
                    "CDPRODUTO"        => $produto["CDPRODUTO"],
                    "__is_new"         => true
                );
                
                try {
                    $this->util->validateDuplicatedRecords($produtoRow, "itlimpesergr", array("CDPRODUTO", "NRGRUPOPRATOSERV", "NRLIMPESERGR"));
                    $dataSet = new DataSet($this->dataSourceName, array($produtoRow));
                    
                    $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                    parent::save($newRequest, $response);
                } catch(\Exception $e) {
                    if ($e->getMessage() == "registerExists") {
                        array_push($this->errorMessages, array("NMPRODUTO" => $produto["NMPRODUTO"]));
                    }
                }
            }

            $this->connection->commit();
            if (!empty($this->errorMessages)) {
                $response->addDataSet(new DataSet("errors", $this->errorMessages));
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    //Importação - Exportação
    public function servicosImportExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();

            $servicos = $row["NMSERVICO"];
            if (isset($row["vigencia"])) {
                $nrvigenciadest = $this->saveVigencia($row["vigencia"], $response);
            } else {
                $nrvigenciadest =  $row["NRVIGENCIA"];
            }
            $tpoper = $row["TPOPER"];

            if ($tpoper == "S") {
                $this->deleteLimpesergrByNrvigencia($nrvigenciadest);
            }
            foreach ($servicos as $servico) {
                $produtos = $this->getItlimpesergr($servico["NRLIMPESERGR"]);
                $nrlimpesergrdest = $this->saveServicoImportExport($servico, $row, $nrvigenciadest);
                $produtosExisting = $this->getItlimpesergr($nrlimpesergrdest);
                $produtosExisting = array_map(function($produto){
                    return $produto["CDPRODUTO"].'|'.$produto["NRGRUPOPRATO"];
                }, $produtosExisting);
                $produtosToSave = array();
                foreach ($produtos as &$produto) {
                    if (in_array($produto["CDPRODUTO"].'|'.$produto["NRGRUPOPRATO"], $produtosExisting)) {
                        continue;
                    } else {
                        $produto["NRLIMPESERGR"] = $nrlimpesergrdest;
                        $produto["NRITLIMPESERGR"] = $this->RestrContrService->getNewCode("ITLIMPESERGR");
                        $produto["__is_new"] = true;
                        array_push($produtosToSave, $produto);
                    }
                }
                $dataSourceName = "itlimpesergr";
                $dataSet = new DataSet($dataSourceName, $produtosToSave);
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
                $dataSet = new DataSet($dataSourceName, $rowsToSave);
                $this->dataSourceManager->persist($dataSet);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function saveServicoImportExport($servico, $vigencia, $nrvigenciadest) {
        $dataSourceName = "limpesergr";

        $conditions = $this->RestrContrService->buildConditions(array("CDSERVFILI", "NRVIGENCIA"), array($servico["CDSERVFILI"], $nrvigenciadest));
        $limpesergr = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        if (!empty($limpesergr)) {
            $nrlimpesergr =  $limpesergr[0]["NRLIMPESERGR"];
        } else {
            $servico["NRVIGENCIA"] = $nrvigenciadest;
            $servico["DTVIGENCIA"] = $vigencia["DTVIGENCIA"];
            $servico["CDFILIAL"] = $vigencia["CDFILIAL"];
            $servico["DSLIMPESERGR"] = $vigencia["DSLIMPESERGR"];
            $nrlimpesergr = $this->RestrContrService->getNewCode("LIMPESERGR");
            $servico["NRLIMPESERGR"] = $nrlimpesergr;
            $servico["__is_new"] = true;
            
            $dataSet = new DataSet($dataSourceName, array($servico));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
        }
        return $nrlimpesergr;
    }

    public function getItlimpesergr($nrlimpesergr) {
        $conditions = $this->RestrContrService->buildConditions(array("NRLIMPESERGR"), array($nrlimpesergr));
        return $this->crudService->getFilteredDataSet("itlimpesergr", $conditions)->getRows();
    }

    private function deleteLimpesergrByNrvigencia($nrvigencia) {
        $sql = "DELETE FROM LIMPESERGR WHERE NRVIGENCIA = :NRVIGENCIA AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public function vigenciaExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();
            $vigenciaOrig = $row["VIGENCIAORIG"];
            $tables = $row["VIGENCIADEST"];
            foreach($tables as $table) {
                if (in_array(strtoupper($table), $this->RestrContrService->getTables())) { // Prevent SQL injection
                    $row = array(
                        "CDFILIAL" => $vigenciaOrig["CDFILIAL"],
                        "DTVIGENCIA" => $vigenciaOrig["DTVIGENCIA"],
                        "DTVIGENCIAFIM" => $vigenciaOrig["DTVIGENCIAFIM"],
                        "DESCRICAO" => $vigenciaOrig["DESCRICAO"],
                        "IDTIPORESTRICAO" => $vigenciaOrig["IDTIPORESTRICAO"],
                        "CDDESCOBSFILIAL" => $vigenciaOrig["CDDESCOBSFILIAL"],
                        "dataSourceName" => strtolower($table),
                        "fieldsToValidate" => array("CDFILIAL", "DTVIGENCIA"),
                        "__is_new" => true
                    );
                    try {
                        $this->RestrContrService->validateRegisterVigencia($row);
                        if (isset($row["DTVIGENCIAFIM"]) && strlen($row["DTVIGENCIAFIM"])) {
                            $this->RestrContrService->validateIntervalOverlap(null, $row["DTVIGENCIA"], $row["DTVIGENCIAFIM"], $row["dataSourceName"], $row["CDFILIAL"], $row["IDTIPORESTRICAO"]);
                        }
                        $this->saveVigencia($row, $response, $row["dataSourceName"]);
                    } catch( \Exception $e) {
                        if ($e->getMessage() == "registerExists") {
                            array_push($this->errors, $table);
                        } else {
                            throw $e;
                        }
                    }
                }
            }
            
            $this->connection->commit();
            if (!empty($this->errors)) {
                $response->addDataSet(new DataSet("errors", $this->errors));
                $response->setError(new Error("vigenciaExportErrors", 500));
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}